@@ -1,15 +1,12 @@
-#!/usr/bin/env python3
-# coding: utf8
-
-# Author: Lenz Furrer, 2020
-
-
 """
 bconv: converter for bio-NLP formats.
 """
 
 
-__version__ = '0.2.1'
+__author__ = "Lenz Furrer"
+
+__version__ = '1.0.0'
 
 
 from .fmt import load, loads, fetch, dump, dumps, LOADERS, FETCHERS, EXPORTERS
+from .doc.document import Collection, Document, Entity, Relation
