@@ -16,7 +16,7 @@ from collections import OrderedDict
 from lxml import etree
 from lxml.builder import E
 
-from ..doc.document import Collection, Document, Entity
+from ..doc.document import Collection, Document, Entity, Relation
 from ._load import CollLoader, text_node, wrap_in_collection
 from ._export import Formatter, XMLMemoryFormatter, StreamFormatter
 from ..util.iterate import peek, json_iterencode
@@ -54,9 +54,9 @@ class _BioCLoader(CollLoader, _OffsetMixin):
         super().__init__()
         self.byte_offsets = byte_offsets
 
-    def collection(self, source, id_):
+    def collection(self, source, id):
         coll_node, docs = self._parse_collection(source)
-        collection = Collection(id_, basename(source))
+        collection = Collection(id, basename(source))
         collection.metadata = self._meta_dict(coll_node)
         for doc in docs:
             collection.add_document(self._document(doc))
@@ -71,19 +71,23 @@ class _BioCLoader(CollLoader, _OffsetMixin):
         """
         doc = Document(self._text(node, 'id', ifnone=None))
         doc.metadata = self.infon_dict(node)
+        doc.relations = self._get_relations(node)
 
         offset_mngr = self._offset_mngr()
         for passage in self._iterfind(node, 'passage'):
             sec_type, text, offset, infon, anno = self._section(passage,
                                                                 offset_mngr)
-            doc.add_section(sec_type, text, offset)
+            doc.add_section(sec_type, text, offset, anno, entity_offset=0)
             section = doc[-1]
             section.metadata = infon
-            section.add_entities(anno)
-            # Get infon elements on sentence level.
+            section.relations = self._get_relations(passage)
+            # Get infon elements and relations at sentence level.
             for sent, sent_node in zip(section,
                                        self._iterfind(passage, 'sentence')):
                 sent.metadata = self.infon_dict(sent_node)
+                sent.relations = self._get_relations(sent_node)
+
+        doc.sanitize_relations()
 
         return doc
 
@@ -92,7 +96,7 @@ class _BioCLoader(CollLoader, _OffsetMixin):
         infon = self.infon_dict(node)
         type_ = infon.get('type')
         text = self._text(node)
-        if text is None:
+        if not text:
             # Text and annotations at sentence level.
             offset = offset_mngr.start(node)
             text, anno = [], []
@@ -102,7 +106,7 @@ class _BioCLoader(CollLoader, _OffsetMixin):
         else:
             # Text and annotations at passage level.
             offset = offset_mngr.update(node, text)
-            anno = self._get_annotations(node, offset_mngr)
+            anno = list(self._get_annotations(node, offset_mngr))
         return type_, text, offset, infon, anno
 
     def _sentence(self, node, offset_mngr):
@@ -113,24 +117,38 @@ class _BioCLoader(CollLoader, _OffsetMixin):
 
     def _get_annotations(self, node, offset_mngr):
         """
-        Iterate over annotations.
-
-        Any non-contiguous annotation is split up into
-        multiple contiguous annotations.
+        Iterate over entity annotations.
         """
         for anno in self._iterfind(node, 'annotation'):
-            for loc in self._iterfind(anno, 'location'):
-                start, length = (int(loc.get(n)) for n in ('offset', 'length'))
-                end = offset_mngr.character(start+length)
-                start = offset_mngr.character(start)
-                yield self._entity(anno, start, end)
+            offsets = list(self._get_offsets(anno, offset_mngr))
+            yield self._entity(anno, offsets)
 
-    def _entity(self, anno, start, end):
+    def _get_offsets(self, node, offset_mngr):
+        for loc in self._iterfind(node, 'location'):
+            start, length = (int(loc.get(n)) for n in ('offset', 'length'))
+            end = offset_mngr.character(start+length)
+            start = offset_mngr.character(start)
+            yield start, end
+
+    def _entity(self, node, offsets):
         """Create an Entity instance from a BioC annotation node."""
-        id_ = anno.get('id')
-        text = self._text(anno)
-        info = self.infon_dict(anno)
-        return Entity(id_, text, start, end, info)
+        id_ = node.get('id')
+        text = self._text(node)
+        entity = Entity(id_, text, offsets)  # don't provide meta as kwargs
+        entity.metadata = self.infon_dict(node)
+        return entity
+
+    def _get_relations(self, node):
+        for relation in self._iterfind(node, 'relation'):
+            yield self._relation(relation)
+
+    def _relation(self, node):
+        id_ = node.get('id')
+        members = ((m.get('refid'), m.get('role', ''))
+                   for m in self._iterfind(node, 'node'))
+        relation = Relation(id_, members)
+        relation.metadata = self.infon_dict(node)
+        return relation
 
     def _meta_dict(self, node):
         """Read metadata into a dictionary."""
@@ -301,6 +319,8 @@ class BioCXMLFormatter(_BioCFormatter, XMLMemoryFormatter, _OffsetMixin):
         for section in doc:
             node.append(self._passage(section, offset_mngr))
 
+        self._add_relations(node, doc.relations)
+
         return node
 
     def _passage(self, section, offset_mngr):
@@ -319,6 +339,8 @@ class BioCXMLFormatter(_BioCFormatter, XMLMemoryFormatter, _OffsetMixin):
                 offset_mngr.sentence(sent)  # synchronise without direct usage
                 self._add_entities(node, sent, offset_mngr)
 
+        self._add_relations(node, section.relations)
+
         return node
 
     def _sentence(self, sent, offset_mngr):
@@ -328,6 +350,7 @@ class BioCXMLFormatter(_BioCFormatter, XMLMemoryFormatter, _OffsetMixin):
         node.append(E('text', sent.text))
 
         self._add_entities(node, sent, offset_mngr)
+        self._add_relations(node, sent.relations)
 
         return node
 
@@ -338,16 +361,27 @@ class BioCXMLFormatter(_BioCFormatter, XMLMemoryFormatter, _OffsetMixin):
     def _entity(self, entity, offset_mngr):
         node = E('annotation', id=str(entity.id))
 
-        for label, value in entity.info.items():
+        for label, value in entity.metadata.items():
             self._infon(node, label, value)
 
-        start, length = offset_mngr.entity(entity)
-        node.append(E('location', offset=str(start), length=str(length)))
+        for start, length in offset_mngr.entity(entity):
+            node.append(E('location', offset=str(start), length=str(length)))
 
         node.append(E('text', entity.text))
 
         return node
 
+    def _add_relations(self, node, relations):
+        for relation in relations:
+            node.append(self._relation(relation))
+
+    def _relation(self, relation):
+        node = E('relation', id=str(relation.id))
+        self._add_meta(node, relation.metadata)
+        for member in relation:
+            node.append(E('node', refid=str(member.refid), role=member.role))
+        return node
+
     def _add_meta(self, node, meta):
         for key, value in meta.items():
             self._infon(node, key, value)
@@ -393,7 +427,7 @@ class BioCJSONFormatter(_BioCFormatter, StreamFormatter, _OffsetMixin):
             'id': str(doc.id),
             'infons': dict(doc.metadata),
             'passages': [self._passage(s, offset_mngr) for s in doc],
-            'relations': (),
+            'relations': list(map(self._relation, doc.relations)),
         }
 
     def _passage(self, section, offset_mngr):
@@ -421,7 +455,7 @@ class BioCJSONFormatter(_BioCFormatter, StreamFormatter, _OffsetMixin):
             'text': text,
             'sentences': sentences,
             'annotations': annotations,
-            'relations': (),
+            'relations': list(map(self._relation, section.relations)),
         }
 
     def _sentence(self, sent, offset_mngr):
@@ -431,17 +465,25 @@ class BioCJSONFormatter(_BioCFormatter, StreamFormatter, _OffsetMixin):
             'text': sent.text,
             'annotations': [self._entity(e, offset_mngr)
                             for e in sent.iter_entities()],
-            'relations': (),
+            'relations': list(map(self._relation, sent.relations)),
         }
 
     @staticmethod
     def _entity(entity, offset_mngr):
-        start, length = offset_mngr.entity(entity)
         return {
             'id': str(entity.id),
-            'infons': dict(entity.info),
+            'infons': dict(entity.metadata),
             'text': entity.text,
-            'locations': [dict(offset=start, length=length)]
+            'locations': [dict(offset=start, length=length)
+                          for start, length in offset_mngr.entity(entity)]
+        }
+
+    @staticmethod
+    def _relation(relation):
+        return {
+            'id': str(relation.id),
+            'infons': dict(relation.metadata),
+            'nodes': [dict(refid=str(m.refid), role=m.role) for m in relation],
         }
 
 
@@ -579,5 +621,6 @@ class ByteOffsetWriter(OffsetWriter, _OffsetConverter):
     _indexer = staticmethod(iter_codepoint_indices_utf8)
 
     def entity(self, entity):
-        start, end = (self.character(n) for n in (entity.start, entity.end))
-        return start, end-start
+        for start, end in entity.spans:
+            start, end = (self.character(n) for n in (start, end))
+            yield start, end-start
