@@ -26,7 +26,7 @@ class _MedlineParser:
     Mixin for parsing PubMed abstracts in Medline's XML format.
     """
 
-    tag = 'MedlineCitation'
+    tag = ('MedlineCitation', 'BookDocument')
 
     def __init__(self, single_section=False,
                  include_mesh=False, mesh_as_entities=False):
@@ -45,10 +45,11 @@ class _MedlineParser:
         doc.metadata = dict(self._get_metadata(node))
 
         # Title.
-        title = ''.join(node.find('.//ArticleTitle').itertext())
-        doc.add_section('Title', title + '\n')
+        title = node.find('.//ArticleTitle')
+        if title is not None:  # title may be missing for BookDocument
+            doc.add_section('Title', ''.join(title.itertext()) + '\n')
 
-        # Abstract body migt contain multiple sections, incl. a MeSH list.
+        # Abstract body might contain multiple sections, incl. a MeSH list.
         sections = self._iter_sections(node)
 
         if self.single_section:
@@ -64,10 +65,16 @@ class _MedlineParser:
 
     @staticmethod
     def _get_metadata(root):
-        date = root.find('.//DateCompleted')
+        date = root.find('.//DateCompleted')  # MedlineCitation
         if date is not None:
             yield 'date', '-'.join(date.find(x).text
                                    for x in ('.//Year', './/Month', './/Day'))
+        else:
+            date = root.find('.//PubDate')  # BookDocument
+            if date is not None:
+                # PubDate can be year-month(-day), year-season, or free text.
+                yield 'date', '-'.join(elem.text for elem in date)
+
         # There may be multiple publication types -- the first one is enough.
         type_ = text_node(root, './/PublicationType')
         if type_:
@@ -113,8 +120,8 @@ class _MedlineParser:
             if label not in ('UNLABELLED', 'Abstract'):
                 flat.append((label + ': ', None))
             if isinstance(text, str):
-                sents = TOKENIZER.tokenize_sentences(text)
-                flat.extend((sent, None) for sent in sents)
+                sents = TOKENIZER.split_sentences(text)
+                flat.extend((sent, None) for sent, _, _ in sents)
             else:
                 # List of MeSH headings.
                 flat.extend(zip(text, anno))
@@ -139,10 +146,9 @@ class _MedlineParser:
                 continue
             id_ = next(counter)
             text = sent.text.rstrip()
-            start = sent.start
-            end = start + len(text)
-            info = {'ui': ui, 'source': 'MeSH'}
-            sent.add_entities((Entity(id_, text, start, end, info),))
+            span = (0, len(text))
+            meta = {'ui': ui, 'source': 'MeSH'}
+            sent.add_entities((Entity(id_, text, [span], meta),))
 
 
 class _PMCParser:
@@ -211,7 +217,8 @@ class _PMCParser:
     @staticmethod
     def _sentence_split(texts):
         for text in texts:
-            yield from TOKENIZER.tokenize_sentences(text)
+            for sent, _, _ in TOKENIZER.split_sentences(text):
+                yield sent
 
     def _itertext(self, node):
         return ''.join(node.itertext()).strip() + self.NL
@@ -254,9 +261,9 @@ class _NCBIFetcher(_IterparseLoader):
             for k, v in [('tool', tool), ('email', email), ('api_key', api_key)]
             if v is not None}
 
-    def load_one(self, source, id_):
+    def load_one(self, source, id):
         # Filename makes no sense for a fetched collection -- delete it.
-        coll = super().load_one(source, id_)
+        coll = super().load_one(source, id)
         coll.filename = None
         return coll
 
