@@ -1,14 +1,11 @@
-#!/usr/bin/env python3
-# coding: utf8
-
-# Author: Lenz Furrer, 2017--2020
-
-
 """
 Streaming utilities.
 """
 
 
+__author__ = "Lenz Furrer"
+
+
 import io
 import os
 import codecs
